/*----------------------------------------------------------*/
// Project name : Reading Indicator
// Author: Mathieu Chartier
// Author website: http://blog.internet-formation.fr
// Project website : http://www.testeur.ml
// Origin: France
// Date: March 5th 2017
// Version: 1.0
// Note: it works perfectly with the last version of jQuery
/*----------------------------------------------------------*/
(function($){
// Fonction de calcul du pourcentage de la lecture d'une zone diverse
function percentReading(readingZone, viewPosition, offsetDistance) {
	/*======================================================*/
	/* Récupération des valeurs générales (hauteur/largeur) */
	/*======================================================*/
	var hauteurFenetre = $(window).innerHeight(); // Hauteur de la fenêtre courante (viewport)
	var hauteurZone = Math.round(readingZone.outerHeight()); // Hauteur de la zone cible
	var largeurZone = Math.round(readingZone.outerWidth()); // Largeur de la zone cible

	// Récupération des positions utiles (offset du scroll, positions de la zone de lecture)
	var positionTop = $(window).scrollTop(); // Position haute au moment du scroll
	var positionBottom = positionTop + hauteurFenetre; // Position basse au moment du scroll

	// Récupération des offsets nécessaires pour les calculs (haut et bas de la zone de lecture)
	var offsetTopZone = Math.round(readingZone.offset().top);

	// Calcule le contexte de lecture (pointeur qui définit la hauteur du pixel de lecture)
	if(viewPosition == "top") {
		var context = positionTop - offsetTopZone;
	} else if(viewPosition == "middle") {
		var context = (positionBottom - (hauteurFenetre / 2)) - offsetTopZone;
	} else if(viewPosition == "bottom") {
		var context = positionBottom - offsetTopZone;
	}

	// Ajoute ou retire un décalage pour le calcul (pour se caler sur la cible d'un oeil, etc.)
	if(offsetDistance != 0) {
		context = context - offsetDistance;
	}

	/*====================================================================*/
	/* Calcul du ratio de la hauteur de la zone de lecture déjà parcourue
	/* Méthode : position "x" de l'écran (top, bottom...) / hauteurZone
	/* 1/ Choisir la position cible (bas, milieu, ajustement...)
	/* 2/ Soustraire le pixel haut ou bas de la zone à la position
	/* 3/ Diviser ce calcul par la hauteur de la zone à lire
	/* 4/ Afficher le résultat en pourcentage
	/*====================================================================*/
	var ratioHauteur = context / hauteurZone;
	var pourcentageHauteur = Math.round(ratioHauteur * 100); // Pourcentage de la hauteur (pour l'affichage)
	var percentageLimit = pourcentageHauteur; // Pourcentage exact total

	// Blocage des chiffres à afficher si le curseur est "hors zone"
	if(context <= 0) {
		pourcentageHauteur = 0; // Affiche 0% avant d'être dans la zone de lecture
	}
	if(context >= hauteurZone) {
		pourcentageHauteur = 100; // Affiche 100% après être sorti de la zone de lecture
	}

	return {
		percent: pourcentageHauteur,
		invertedPercent: 100 - pourcentageHauteur,
		realPercent: percentageLimit,
		ratioContext: context,
		ratioHeight: ratioHauteur
	}
}

// Fonction de "debounce"
function debounceScroll(cbck, delay) {
	var timeout; // Instanciation de la variable
	return function() {
		var _this = this; // Élément courant
		var args = arguments; // Liste des arguments

		// Effacement du timeout
		if(timeout) {
			clearTimeout(timeout);
		}

		// Lancement du timeout
		timeout = setTimeout(function() {
			cbck.apply(_this, args);
		}, delay);
	};
};

var count = 0; // Pour l'usage multiple du plugin
$.fn.readingIndicator = function(args) {
	return this.each(function() {
		count = count + 1; // Incrémentation pour l'usage multiple du plugin

		// Options par défaut ou non
		var options = $.extend({}, {
			context: 'middle',							// Contexte du pointeur pour le calcul (top / middle / bottom)
			distance: 0,								// Décale la position pour le calcul (-n pour remonter, sinon n)
			targetProgressBar: 'body',					// Zone cible pour afficher la barre de défilement
			fixedBarSize: false,						// Hauteur/Largeur barre = Hauteur/Largeur zone à lire
			stickyProgressBar: true,					// Rendre la barre "collante" (sticky)
			stickyClass: 'stickyProgressBar',			// Classe pour le mode sticky (barre fixée à l'écran)
			progressPositions: {
				progressBarPosition: 'top',				// Position de la barre (top/bottom/left/right), false pour masquer
				progressValuePosition: 'bottom',		// Position de la valeur (top/bottom/left/right), false pour annuler
			},
			progressBarValue: true,						// Active l'affichage du pourcentage
			progressBarID: 'reading-scroller',			// ID du bloc affichant le pourcentage
			progressBarClass: 'reading-progress',		// Classe du bloc qui dessine la progression
			progressPercentClass: 'reading-percentage',	// Classe du bloc qui affiche le pourcentage
			percentSignClass: 'reading-sign',			// Classe du signe % (pour le décaler en CSS, etc.)
			hideBeforePercent: false,					// Pourcentage avant lequel on masque la barre
			hideAfterPercent: false,					// Pourcentage après lequel on masque la barre
			hideBeforeOffsetTop: false,					// Masquage de la barre avant l'offset Top de la zone de lecture
			hideAfterOffsetBottom: false,				// Masquage de la barre après l'offset Bottom de la zone de lectur
			scrollStopDelay: 200,						// Délai (en ms) pour l'event scrollStop et le debouncing
			cbkOnce: false,								// Pour n'activer la fonction callback qu'une fois (fired)
			cbk: function() {},							// Fonction de rappel debounce (pourcentage, pourcentage réel)
			cbkDisplay: function() {},					// Fonction de rappel pour personnaliser l'affichage
			cbkPercentDisplay: function() {},			// Fonction de rappel pour personnaliser l'affichage du pourcentage
		}, args);

		// Récupération de la zone cible
		var $this = $(this);

		// Pour dissocier les sélecteurs (si multiple instance)
		var el = $this.find('.'+options.progressBarID+'-'+count);
		var elmt = el.selector;

		// Récupération des classes (pour les fonctions de callback)
		var cssClasses = {
			progressBarID: "#"+options.progressBarID,
			progressBarClass: elmt,
			progressPercentClass: "."+options.progressPercentClass,
			percentSignClass: "."+options.percentSignClass,
		};

		/*============================================*/
		/* Création de la barre de défilement adaptée */
		/*============================================*/
		// Création du bloc pour afficher le pourcentage
		$(options.targetProgressBar).append('<div id="'+options.progressBarID+'" class="'+options.progressBarID+'-'+count+'"><div class="'+options.progressBarClass+'"><div class="'+options.progressPercentClass+'"></div></div></div>');
		$("#"+options.progressBarID).hide(); // On masque la barre par défaut

		// Callback pour l'affichage complet de la barre
		$(elmt).html(options.cbkDisplay.call($(elmt), cssClasses));

		// Position:relative pour le bloc d'origine
		$($this).css({position:'relative'});
		$(options.targetProgressBar).css({position:'relative'});

		// Ajoute la classe CSS utile selon la position de la barre choisie
		if(jQuery.inArray(options.progressPositions.progressBarPosition, ["top", "bottom", "left", "right"]) != -1) {
			$(elmt).addClass("reading-"+options.progressPositions.progressBarPosition);
		}
		if(options.progressPositions.progressBarPosition == false) {
			$(elmt).hide();
		}

		/*=================================================*/
		/* Effet et calcul du pourcentage en cas de scroll */
		/*=================================================*/
		var scrollerPercent = debounceScroll(function(e) {
			// Calcule du pourcentage de lecture
			var measure = percentReading($this, options.context, options.distance);

			// Récupération des valeurs utiles (objet JSON)
			var pourcentageHauteur = measure.percent; // Pourcentage de 0% à 100%
			var pourcentageInverse = measure.invertedPercent; // Pourcentage inversé (reste "x" %)
			var percentageLimit = measure.realPercent; // Pourcentage réel
			var context = measure.ratioContext; // Contexte de calcul
			var ratioHauteur = measure.ratioHeight; // Contexte de calcul

			// Ajout des valeurs dans des attributs (au cas où...)
			$(elmt).attr('data-real-progress', percentageLimit);
			$(elmt).attr('data-inverted-progress', pourcentageInverse);
			$(elmt).attr('data-progress', pourcentageHauteur);

			/*======================================================*/
			/* Récupération des valeurs générales (hauteur/largeur) */
			/*======================================================*/
			var hauteurFenetre = $(window).height(); // Hauteur de la fenêtre courante (viewport)
			var hauteurZone = Math.round($this.height()); // Hauteur de la zone cible
			var largeurZone = Math.round($this.width()); // Largeur de la zone cible

			// Récupération des positions utiles (offset du scroll, positions de la zone de lecture)
			var positionTop = $(window).scrollTop(); // Position haute au moment du scroll
			var positionBottom = positionTop + hauteurFenetre; // Position basse au moment du scroll

			/*=========================================================*/
			/* Gestion de la barre de défilement adaptée + pourcentage */
			/*=========================================================*/
			if($.inArray(options.progressPositions.progressBarPosition, ["top", "bottom", "left", "right"]) != -1) {
				// Masquage de la barre de chargement selon les limites fixées
				if(options.hideBeforePercent !== false || options.hideAfterPercent !== false || options.hideBeforeOffsetTop !== false || options.hideAfterOffsetBottom !== false) {
					// Annule en cas de doublon de masquage
					if(options.hideBeforePercent !== false && options.hideBeforeOffsetTop == true) {
						options.hideBeforePercent = false;
					}
					if(options.hideAferPercent !== false && options.hideAfterOffsetTop == true) {
						options.hideAfterPercent = false;
					}

					// Gère les variables showX
					if((options.hideBeforePercent !== false && percentageLimit <= options.hideBeforePercent) || (options.hideBeforeOffsetTop == true && context <= 0)) {
						var showBefore = false;
					}
					if((options.hideAfterPercent !== false && percentageLimit >= options.hideAfterPercent) || (options.hideAfterOffsetBottom == true && context >= hauteurZone)) {
						var showAfter = false;
					}

					// Affiche ou non la barre
					if(showBefore == false || showAfter == false) {
						$(elmt).fadeOut();
					} else {
						$(elmt).fadeIn();
					}
				} else {
					$(elmt).show(); // Affiche la barre si aucune option ne prétend le contraire
				}

				// Fixe la taille du bloc (hauteur ou largeur à 100% de la taille de la zone à lire)
				if(options.fixedBarSize == true || options.targetProgressBar != 'body') {
					// Adaptation selon l'orientation de la taille de la barre de défilement
					if(options.progressPositions.progressBarPosition == "left" || options.progressPositions.progressBarPosition == "right") {
						$(elmt).height(hauteurZone);
					}
					if(options.progressPositions.progressBarPosition == "top" || options.progressPositions.progressBarPosition == "bottom") {
						$(elmt).width(largeurZone);
					}
				}

				// CSS de la barre de défilement selon l'orientation
				var progressBarCSS = ""; // Vide par défaut (si mauvaise valeur)
				if(options.progressPositions.progressBarPosition == "left" || options.progressPositions.progressBarPosition == "right") {
					var progressBarCSS = {"height": pourcentageHauteur+"%"};
				}
				if(options.progressPositions.progressBarPosition == "top" || options.progressPositions.progressBarPosition == "bottom") {
					var progressBarCSS = {"width": pourcentageHauteur+"%"};
				}
				$(elmt+" ."+options.progressBarClass).css(progressBarCSS);

				// Callback pour la fonction d'affichage du pourcentage uniquement
				if(options.progressBarValue == true || options.progressBarPosition.progressValuePosition == false) {
					// Gère l'affichage du pourcentage uniquement (avec ou sans callback)
					if($.isEmptyObject(options.cbkPercentDisplay.call($(elmt), measure, cssClasses))) {
						$(elmt+" ."+options.progressPercentClass).html(pourcentageHauteur+'<span class="'+options.percentSignClass+'">%</span>');
					} else if(!$.isEmptyObject(options.cbkPercentDisplay.call($(elmt), measure, cssClasses))) {
						// Gestion de l'affichage avec une fonction de rappel (callback)
						$(elmt+" ."+options.progressPercentClass).html(options.cbkPercentDisplay.call($(elmt), measure, cssClasses));
					}

					// Ajoute la classe pour positionner la valeur (si affichée => donc pas false)
					if($.inArray(options.progressPositions.progressValuePosition, ["top", "bottom", "left", "right"]) != -1) {
						$(elmt+' .'+options.progressPercentClass).addClass("value-"+options.progressPositions.progressValuePosition);
					}
				}

				// Dans le cas où la barre doit se fixer en haut de la zone de lecture (sticky)
				if(options.stickyProgressBar == true || options.targetProgressBar == 'body') {
					if((options.progressPositions.progressBarPosition == "top" && (positionTop >= Math.round($(options.targetProgressBar).offset().top - $(elmt).height() - $(elmt+' .'+options.progressPercentClass).height()))) ||
						(options.progressPositions.progressBarPosition == "bottom" && (positionBottom >= Math.round($(options.targetProgressBar).offset().top))) ||
						(options.progressPositions.progressBarPosition == "left" || options.progressPositions.progressBarPosition == "right" && options.targetProgressBar == 'body')) {
						$(elmt).addClass(options.stickyClass);
					} else {
						$(elmt).removeClass(options.stickyClass);
					}
				}
			}

			/*=========================================*/
			/* Création d'un custom event : scrollStop */
			/*=========================================*/
			// e.getProgress pour obtenir le pourcentage (0% à 100%)
			// e.getRealProgress pour obtenir le pourcentage réel (< 0% à > 100%)
			$(elmt).trigger({
				type: "scrollStop",
				getProgress: pourcentageHauteur,
				getInvertedProgress: pourcentageInverse,
				getRealProgress: percentageLimit,
				getEvent: e.type
			});

			// Evite la propagation de l'effet
			e.stopPropagation();
			e.preventDefault();
  		}, options.scrollStopDelay);

		/*======================================================*/
		/* Activation de la fonction au chargement ou au scroll */
		/*======================================================*/
  		$(window).on('load scroll', scrollerPercent);

  		/*====================================================*/
		/* Lancement d'une fonction CallBack en fin de scroll */
		/*====================================================*/
  		var reachedScroll = false;
		$(elmt).on('scrollStop', function(e) {
			if(options.cbkOnce === true) {
				if(!el.reachedScroll) {
					options.cbk.call($(elmt), e.getProgress, e.getRealProgress, e.getInvertedProgress);
					if(e.getEvent == "load") {
						el.reachedScroll = true;
					}
				}
			} else {
				options.cbk.call($(elmt), e.getProgress, e.getRealProgress, e.getInvertedProgress);
			}
		});
	});
};
})(jQuery);